' Ron Charlton explicitly places 'Demo.vb' in the public domain.
' It may be used for any purpose.
'
' $Id: Demo64.vb 1.7 2014-06-18 13:09:18-04 Ron Exp $ $Name: release1dot7 $

' The following #Const must match the one in file mt19937-64.vb.
' True if you use BigInteger with VB 2010 or later;
' False otherwise:
#Const Need128bitInts = False

Option Strict On
Option Explicit On

Imports System
Imports System.IO
Imports System.Text ' for Encoding.UTF8
#If Need128bitInts Then
Imports System.Numerics ' for BigInteger
#End If

Module Demonstration64
	' This module demonstrates the Visual Basic .NET MTRandom64 class.
	' Author: Ron Charlton
	' Date:   2012-08-16

	Public Sub Demo64()
		Const demoOutputFileName As String = "Demo64.out.txt"

		'--------------------------------
		' There are eight ways to declare and initialize an MTRandom64 pseudo-
		' random number generator (PRNG).  Here are examples:
		'--------------------------------

		' 1) Declare a PRNG with default initialization.
		'    The default seed is 5489UL (as in the original MT authors' version).
		Dim a As New MTRandom64()
		'	  -- OR --
		' Dim a As New MTRandom64

		' 2) Declare another PRNG with seed == 5489UL (the same as the default seed).
		'    Any integer seed in the range [0,] is acceptable.
		Dim b As New MTRandom64(5489UL)

		' 3) Declare a third PRNG and initialize it with an array of integers
		'	 (as in the original MT authors' test).
		Dim init64() As ULong = {&H12345UL, &H23456UL, &H34567UL, &H45678UL}
		Dim c As New MTRandom64(init64)

		' 4) Declare a fourth PRNG and initialize it randomly based on the
		'	 system clock (True is required but ignored)
		Dim d As New MTRandom64(True)

		' 5) Declare a fifth PRNG and initialize it with a generator state
		'	 that was saved in file "myfile.xml" with MTRandom64.saveState().
		'Dim e As New MTRandom64("myfile.xml")	' (no such file, so commented out)

		' 6) Declare a sixth PRNG and initialize it with good pseudorandom numbers.
		'	 The Double argument is required but ignored.
		Dim f As New MTRandom64(0.0)

		' 7) Declare a seventh PRNG and initialize it with 32-bit, unsigned integers.
		Dim init32() As UInteger = {&H12345UI, 0, &H23456UI, 0, &H34567UI, 0, &H45678UI, 0}
		Dim g As New MTRandom64(init32)

		' 7) Declare an eighth PRNG and initialize it with generator f's state.
		Dim h As New MTRandom64(f)

		' send output to a file
		Using sw As StreamWriter = File.CreateText(demoOutputFileName)

			sw.WriteLine("Demo64.vb output:")

			' make sure initialization of h from f works
			If f = h Then
				sw.WriteLine("Generator h's state is the same as generator f's state.")
			End If

			If f <> h Then
				sw.WriteLine("ERROR: Generator h's state is NOT the same as generator f's state.")
			End If

			'--------------------------------
			' Generate some pseudorandom numbers:
			'--------------------------------
			' Generate random integers in the range [0,18446744073709551615]
			' (0 to 2^64-1).
			Dim x As ULong = a.genrand64_int64()
			Dim y As ULong = b.genrand64_int64()

			' Show that PRNGs a and b produced the same initial value, because each has
			' its own generator state and they have the same initial seed.
			sw.WriteLine("The following two numbers should be equal...")
			sw.WriteLine("x: {0}", x)
			sw.WriteLine("y: {0}", y)
			If x <> y Then
				sw.WriteLine("a.genrand64_int64() and b.genrand64_int64() gave different results.")
			Else
				sw.WriteLine("   ...They are.")
			End If

			'--------------------------------
			' Reinitialize the PRNGs. (A different initialization method
			' than used originally is acceptable on any generator.)
			'--------------------------------
			a.init_genrand64(&HDD9CC370UL)
			c.init_genrand64(1234987644UL)
			d.init_by_random64(False)		' use the next initializer random number
			'							    to re-initialize MTRandom64
			b.init_by_crypto64(0.0)		' restart with a cryptographically random
			'                               generator state

			'--------------------------------
			' Save b's generator state to a file named "DemoState64.xml", use b some,
			' then restore b's generator to its saved state.
			'--------------------------------
			Dim loopCount As Integer

			b.init_by_array64(init64)
			b.saveState64("DemoState64.xml")
			For loopCount = 1 To 1000000
				b.genrand64_int64()
			Next
			b.loadState64("DemoState64.xml")	' restore the existing PRNG's saved state
			'	-- OR --
			'b = New MTRandom64("DemoState64.xml")	' make an entirely new PRNG instance

			' Demonstrate the comparison operator "=" (equal).
			Dim newb As New MTRandom64(init64)
			If newb = b Then
				sw.WriteLine("Generator b's state WAS saved and loaded correctly.")
			Else
				sw.WriteLine("ERROR: Generator b's state WAS NOT saved and loaded correctly.")
			End If

			' Test initialization with a UInteger array.
			Dim h64 As New MTRandom64(init64)
			Dim h32 As New MTRandom64(init32)
			If h32 = h64 Then
				sw.WriteLine("Generator h32's state WAS initialized correctly.")
			Else
				sw.WriteLine("ERROR: Generator h32's state was NOT initialized correctly.")
			End If

			' Demonstrate the comparison operator "<>" (not equal).
			sw.WriteLine("Use b to generate a random number: {0}.", b.genrand64_int64())
			If newb <> b Then
				sw.WriteLine("Generator b's state has changed, as expected.")
			Else
				sw.WriteLine("Generator b's state has NOT changed.")
			End If
			sw.WriteLine("Is b's state different from newb's state?: {0}", newb <> b)

			'--------------------------------
			' Call each of the MTRandom64 functions and show the result.
			'--------------------------------
			Dim Dbl As Double
			Dim ULng As ULong

			ULng = a.genrand64_int64()
			sw.WriteLine("A ULong in range [0,18446744073709551615]: {0}", ULng)

			Dim Lng As Long = a.genrand64_int63()
			sw.WriteLine("A Long in range [0,9223372036854775807] (0 to 2^63-1): {0}", Lng)

			Dbl = a.genrand64_real1()
			sw.WriteLine("A Double in range [0.0,1.0] (both 0.0 and 1.0 included): {0}", Dbl)

			Dbl = a.genrand64_real2()
			sw.WriteLine("A Double in range [0.0,1.0) (0.0 included, 1.0 excluded): {0}", Dbl)

			Dbl = a.genrand64_real3()
			sw.WriteLine("A Double in range (0.0,1.0) (both 0.0 and 1.0 excluded): {0}", Dbl)

			ULng = a.genrand64_intMax(100)
			sw.WriteLine("A ULong in range [0,100]: {0}", ULng)

			ULng = a.genrand64_intRange(0, 8000)
			sw.WriteLine("A ULong in range [0,8000]: {0}", ULng)

#If Need128bitInts Then
			Dim bigInt As BigInteger = a.genrand64_int128SignedInt()
			sw.WriteLine("A BigInteger in range [-2^127,2^127-1]: {0}", bigInt.ToString("D"))
#End If

			' Display the crypto values in crypto.state64.mt(), four to a line
			sw.WriteLine("state64.mt() from f.init_by_crypto64(0.0):")
			Dim n As Integer, s As String
			For n = 0 To f.state64.mt.GetUpperBound(0)
				''''sw.Write("{0,16:X} ", f.state64.mt(n))	' won't write leading zeroes
				' Stupid VB type conversions won't write leading zeroes in hex
				s = Replace(String.Format("{0,16:X}", f.state64.mt(n)), " ", "0")
				sw.Write("{0} ", s)
				If n Mod 4 = 3 Then
					sw.WriteLine("")
				End If
			Next n

		End Using

		' read the file just written and display it on the console
		Console.WriteLine(File.ReadAllText(demoOutputFileName, Encoding.UTF8))

		' say "file was written" on the console
		Console.WriteLine("")
		Console.WriteLine("Demo.vb output was also written to file ""{0}"".", demoOutputFileName)
		Console.WriteLine("Press the <Enter> key to continue.")
		Console.ReadLine()
	End Sub	  ' Demo64
End Module
